@@ -53,36 +53,39 @@
 
 
 
-static void               terminal_app_finalize                 (GObject            *object);
-static void               terminal_app_update_accels            (TerminalApp        *app);
-static void               terminal_app_update_mnemonics         (TerminalApp        *app);
-static gboolean           terminal_app_accel_map_load           (gpointer            user_data);
-static gboolean           terminal_app_accel_map_save           (gpointer            user_data);
-static void               terminal_app_new_window               (TerminalWindow     *window,
-                                                                 const gchar        *working_directory,
-                                                                 TerminalApp        *app);
-static void               terminal_app_new_window_with_terminal (TerminalWindow     *window,
-                                                                 TerminalScreen     *terminal,
-                                                                 gint                x,
-                                                                 gint                y,
-                                                                 TerminalApp        *app);
-static void               terminal_app_window_destroyed         (GtkWidget          *window,
-                                                                 TerminalApp        *app);
-static void               terminal_app_save_yourself            (XfceSMClient       *client,
-                                                                 TerminalApp        *app);
-static void               terminal_app_open_window              (TerminalApp        *app,
-                                                                 TerminalWindowAttr *attr);
+static void     terminal_app_finalize                 (GObject            *object);
+static void     terminal_app_update_accels            (TerminalApp        *app);
+static void     terminal_app_update_mnemonics         (TerminalApp        *app);
+static gboolean terminal_app_accel_map_load           (gpointer            user_data);
+static gboolean terminal_app_accel_map_save           (gpointer            user_data);
+static void     terminal_app_unset_urgent_bell        (TerminalWindow     *window,
+                                                       GdkEvent           *event,
+                                                       TerminalApp        *app);
+static void     terminal_app_new_window               (TerminalWindow     *window,
+                                                       const gchar        *working_directory,
+                                                       TerminalApp        *app);
+static void     terminal_app_new_window_with_terminal (TerminalWindow     *window,
+                                                       TerminalScreen     *terminal,
+                                                       gint                x,
+                                                       gint                y,
+                                                       TerminalApp        *app);
+static void     terminal_app_window_destroyed         (GtkWidget          *window,
+                                                       TerminalApp        *app);
+static void     terminal_app_save_yourself            (XfceSMClient       *client,
+                                                       TerminalApp        *app);
+static void     terminal_app_open_window              (TerminalApp        *app,
+                                                       TerminalWindowAttr *attr);
 
 
 
 struct _TerminalAppClass
 {
-  GObjectClass  __parent__;
+  GObjectClass parent_class;
 };
 
 struct _TerminalApp
 {
-  GObject              __parent__;
+  GObject              parent_instance;
   TerminalPreferences *preferences;
   XfceSMClient        *session_client;
   gchar               *initial_menu_bar_accel;
@@ -127,6 +130,8 @@ terminal_app_init (TerminalApp *app)
   app->preferences = terminal_preferences_get ();
   g_signal_connect_swapped (G_OBJECT (app->preferences), "notify::shortcuts-no-menukey",
                             G_CALLBACK (terminal_app_update_accels), app);
+  g_signal_connect_swapped (G_OBJECT (app->preferences), "notify::shortcuts-no-helpkey",
+                            G_CALLBACK (terminal_app_update_accels), app);
   g_signal_connect_swapped (G_OBJECT (app->preferences), "notify::shortcuts-no-mnemonics",
                             G_CALLBACK (terminal_app_update_mnemonics), app);
 
@@ -165,6 +170,8 @@ terminal_app_finalize (GObject *object)
     {
       g_signal_handlers_disconnect_by_func (G_OBJECT (lp->data), G_CALLBACK (terminal_app_window_destroyed), app);
       g_signal_handlers_disconnect_by_func (G_OBJECT (lp->data), G_CALLBACK (terminal_app_new_window), app);
+      g_signal_handlers_disconnect_by_func (G_OBJECT (lp->data), G_CALLBACK (terminal_app_new_window_with_terminal), app);
+      g_signal_handlers_disconnect_by_func (G_OBJECT (lp->data), G_CALLBACK (terminal_app_unset_urgent_bell), app);
       gtk_widget_destroy (GTK_WIDGET (lp->data));
     }
   g_slist_free (app->windows);
@@ -187,15 +194,21 @@ terminal_app_finalize (GObject *object)
 static void
 terminal_app_update_accels (TerminalApp *app)
 {
-  gboolean no_menukey;
+  gboolean no_key;
 
   g_object_get (G_OBJECT (app->preferences),
-                "shortcuts-no-menukey", &no_menukey,
+                "shortcuts-no-menukey", &no_key,
                 NULL);
   g_object_set (G_OBJECT (gtk_settings_get_default ()),
                 "gtk-menu-bar-accel",
-                no_menukey ? NULL : app->initial_menu_bar_accel,
+                no_key ? NULL : app->initial_menu_bar_accel,
                 NULL);
+
+  g_object_get (G_OBJECT (app->preferences),
+                "shortcuts-no-helpkey", &no_key,
+                NULL);
+  gtk_accel_map_change_entry ("<Actions>/terminal-window/contents",
+                              no_key ? 0 : GDK_KEY_F1, 0, TRUE);
 }
 
 
@@ -257,6 +270,7 @@ static gboolean
 terminal_app_accel_map_load (gpointer user_data)
 {
   TerminalApp *app = TERMINAL_APP (user_data);
+  GtkAccelKey  key;
   gchar       *path;
   gchar        name[50];
   guint        i;
@@ -270,26 +284,46 @@ terminal_app_accel_map_load (gpointer user_data)
       gtk_accel_map_load (path);
       g_free (path);
     }
-  else
-    {
-      /* create default Alt+N accelerators */
-      for (i = 1; i < 10; i++)
-        {
-          g_snprintf (name, sizeof (name), "<Actions>/terminal-window/goto-tab-%d", i);
-          gtk_accel_map_change_entry (name, GDK_KEY_0 + i, GDK_MOD1_MASK, FALSE);
-        }
-    }
 
   /* watch for changes */
   app->accel_map = gtk_accel_map_get ();
   g_signal_connect_swapped (G_OBJECT (app->accel_map), "changed",
       G_CALLBACK (terminal_app_accel_map_changed), app);
 
+  /* check and create default Alt+N accelerators */
+  for (i = 1; i < 10; i++)
+    {
+      g_snprintf (name, sizeof (name), "<Actions>/terminal-window/goto-tab-%d", i);
+      if (!gtk_accel_map_lookup_entry (name, &key) || key.accel_key == 0)
+        gtk_accel_map_change_entry (name, GDK_KEY_0 + i, GDK_MOD1_MASK, FALSE);
+    }
+
   return FALSE;
 }
 
 
 
+static void
+terminal_app_unset_urgent_bell (TerminalWindow *window,
+                                GdkEvent       *event,
+                                TerminalApp    *app)
+{
+  GtkWidget *toplevel;
+  gboolean   enabled;
+
+  g_object_get (G_OBJECT (app->preferences),
+                "misc-bell-urgent", &enabled,
+                NULL);
+
+  if (!enabled)
+    return;
+
+  toplevel = gtk_widget_get_toplevel (GTK_WIDGET (window));
+  gtk_window_set_urgency_hint (GTK_WINDOW (toplevel), FALSE);
+}
+
+
+
 static void
 terminal_app_take_window (TerminalApp *app,
                           GtkWindow   *window)
@@ -308,6 +342,10 @@ terminal_app_take_window (TerminalApp *app,
                     G_CALLBACK (terminal_app_new_window), app);
   g_signal_connect (G_OBJECT (window), "new-window-with-screen",
                     G_CALLBACK (terminal_app_new_window_with_terminal), app);
+  g_signal_connect (G_OBJECT (window), "focus-in-event",
+                    G_CALLBACK (terminal_app_unset_urgent_bell), app);
+  g_signal_connect (G_OBJECT (window), "key-release-event",
+                    G_CALLBACK (terminal_app_unset_urgent_bell), app);
   app->windows = g_slist_prepend (app->windows, window);
 }
 
@@ -412,6 +450,7 @@ terminal_app_new_window_with_terminal (TerminalWindow *existing,
 {
   GtkWidget *window;
   GdkScreen *screen;
+  glong      width, height;
 
   terminal_return_if_fail (TERMINAL_IS_WINDOW (existing));
   terminal_return_if_fail (TERMINAL_IS_SCREEN (terminal));
@@ -440,6 +479,10 @@ terminal_app_new_window_with_terminal (TerminalWindow *existing,
 
   terminal_window_add (TERMINAL_WINDOW (window), terminal);
 
+  /* resize new window to the original terminal geometry */
+  terminal_screen_get_size (terminal, &width, &height);
+  terminal_screen_force_resize_window (terminal, GTK_WINDOW (window), width, height);
+
   gtk_widget_show (window);
 }
 
@@ -623,6 +666,8 @@ terminal_app_open_window (TerminalApp        *app,
   TerminalScreen  *active_terminal;
   gint             mask = NoValue, x, y;
   guint            width, height;
+  gint             screen_width, screen_height;
+  gint             window_width, window_height;
 #endif
 
   terminal_return_if_fail (TERMINAL_IS_APP (app));
@@ -768,10 +813,21 @@ terminal_app_open_window (TerminalApp        *app,
             {
               active_terminal = terminal_window_get_active (TERMINAL_WINDOW (window));
               if (G_LIKELY (active_terminal != NULL))
-                terminal_screen_set_size (active_terminal, width, height);
+                terminal_screen_force_resize_window (active_terminal, GTK_WINDOW (window),
+                                                     width, height);
             }
           if ((mask & XValue) && (mask & YValue))
-            gtk_window_move (GTK_WINDOW (window), x, y);
+            {
+              screen = gtk_window_get_screen (GTK_WINDOW (window));
+              screen_width = gdk_screen_get_width (screen);
+              screen_height = gdk_screen_get_height (screen);
+              gtk_window_get_default_size (GTK_WINDOW (window), &window_width, &window_height);
+              if (mask & XNegative)
+                x = screen_width - window_width + x;
+              if (mask & YNegative)
+                y = screen_height - window_height + y;
+              gtk_window_move (GTK_WINDOW (window), x, y);
+            }
         }
       else
 #else
