@@ -76,6 +76,7 @@ enum
   PROP_FONT_NAME,
   PROP_MISC_ALWAYS_SHOW_TABS,
   PROP_MISC_BELL,
+  PROP_MISC_BELL_URGENT,
   PROP_MISC_BORDERS_DEFAULT,
   PROP_MISC_CURSOR_BLINKS,
   PROP_MISC_CURSOR_SHAPE,
@@ -91,6 +92,7 @@ enum
   PROP_MISC_TAB_POSITION,
   PROP_MISC_HIGHLIGHT_URLS,
   PROP_MISC_MIDDLE_CLICK_OPENS_URI,
+  PROP_MISC_DEFAULT_WORKING_DIR,
   PROP_SCROLLING_BAR,
   PROP_SCROLLING_LINES,
   PROP_SCROLLING_ON_OUTPUT,
@@ -108,18 +110,23 @@ enum
   N_PROPERTIES,
 };
 
+struct _TerminalPreferencesClass
+{
+  GObjectClass parent_class;
+};
+
 struct _TerminalPreferences
 {
-  GObject __parent__;
+  GObject       parent_instance;
 
-  GValue           values[N_PROPERTIES];
+  GValue        values[N_PROPERTIES];
 
-  GFile           *file;
-  GFileMonitor    *monitor;
-  guint32          last_mtime;
+  GFile        *file;
+  GFileMonitor *monitor;
+  guint64       last_mtime;
 
-  guint            store_idle_id;
-  guint            loading_in_progress : 1;
+  guint         store_idle_id;
+  guint         loading_in_progress : 1;
 };
 
 
@@ -654,6 +661,16 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
                             FALSE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * TerminalPreferences:misc-bell-urgent:
+   **/
+  preferences_props[PROP_MISC_BELL_URGENT] =
+      g_param_spec_boolean ("misc-bell-urgent",
+                            NULL,
+                            "MiscBellUrgent",
+                            FALSE,
+                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   /**
    * TerminalPreferences:misc-borders-default:
    **/
@@ -806,6 +823,16 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
                             FALSE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * TerminalPreferences:misc-default-working-dir:
+   **/
+  preferences_props[PROP_MISC_DEFAULT_WORKING_DIR] =
+      g_param_spec_string ("misc-default-working-dir",
+                           NULL,
+                           "MiscDefaultWorkingDir",
+                           "",
+                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   /**
    * TerminalPreferences:scrolling-bar:
    **/
