@@ -44,7 +44,6 @@
 #include <terminal/terminal-enum-types.h>
 #include <terminal/terminal-image-loader.h>
 #include <terminal/terminal-marshal.h>
-#include <terminal/terminal-private.h>
 #include <terminal/terminal-screen.h>
 #include <terminal/terminal-widget.h>
 #include <terminal/terminal-window.h>
@@ -93,8 +92,6 @@ static void       terminal_screen_set_property                  (GObject
                                                                  GParamSpec            *pspec);
 static void       terminal_screen_realize                       (GtkWidget             *widget);
 static void       terminal_screen_unrealize                     (GtkWidget             *widget);
-static gboolean   terminal_screen_key_press_event               (GtkWidget             *widget,
-                                                                 GdkEventKey           *event);
 static void       terminal_screen_preferences_changed           (TerminalPreferences   *preferences,
                                                                  GParamSpec            *pspec,
                                                                  TerminalScreen        *screen);
@@ -144,16 +141,19 @@ static void       terminal_screen_update_label_orientation      (TerminalScreen
 static gchar     *terminal_screen_zoom_font                     (TerminalScreen        *screen,
                                                                  gchar                 *font_name,
                                                                  TerminalZoomLevel      zoom);
+static void       terminal_screen_urgent_bell                   (TerminalWidget        *widget,
+                                                                 TerminalScreen        *screen);
+
 
 
 struct _TerminalScreenClass
 {
-  GtkHBoxClass __parent__;
+  GtkHBoxClass parent_class;
 };
 
 struct _TerminalScreen
 {
-  GtkHBox              __parent__;
+  GtkHBox              parent_instance;
   TerminalPreferences *preferences;
   GtkWidget           *terminal;
   GtkWidget           *scrollbar;
@@ -205,7 +205,6 @@ terminal_screen_class_init (TerminalScreenClass *klass)
   gtkwidget_class = GTK_WIDGET_CLASS (klass);
   gtkwidget_class->realize = terminal_screen_realize;
   gtkwidget_class->unrealize = terminal_screen_unrealize;
-  gtkwidget_class->key_press_event = terminal_screen_key_press_event;
 
   /**
    * TerminalScreen:custom-title:
@@ -456,23 +455,6 @@ terminal_screen_unrealize (GtkWidget *widget)
 
 
 
-static gboolean
-terminal_screen_key_press_event (GtkWidget   *widget,
-                                 GdkEventKey *event)
-{
-  gboolean ret;
-  TerminalScreen *screen = TERMINAL_SCREEN (widget);
-
-  terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
-
-  /* propagate to the terminal */
-  g_signal_emit_by_name (G_OBJECT (screen->terminal), "key-press-event", event, &ret);
-
-  return ret;
-}
-
-
-
 static void
 terminal_screen_preferences_changed (TerminalPreferences *preferences,
                                      GParamSpec          *pspec,
@@ -498,7 +480,7 @@ terminal_screen_preferences_changed (TerminalPreferences *preferences,
     terminal_screen_update_colors (screen);
   else if (strncmp ("font-", name, strlen ("font-")) == 0)
     terminal_screen_update_font (screen);
-  else if (strcmp ("misc-bell", name) == 0)
+  else if (strncmp ("misc-bell", name, strlen ("misc-bell")) == 0)
     terminal_screen_update_misc_bell (screen);
   else if (strcmp ("misc-cursor-blinks", name) == 0)
     terminal_screen_update_misc_cursor_blinks (screen);
@@ -747,7 +729,8 @@ terminal_screen_get_child_environment (TerminalScreen *screen)
   if (toplevel != NULL && gtk_widget_get_realized (toplevel))
     {
 #ifdef GDK_WINDOWING_X11
-      result[n++] = g_strdup_printf ("WINDOWID=%ld", (glong) gdk_x11_window_get_xid (gtk_widget_get_window (toplevel)));
+      if (GDK_IS_X11_WINDOW (gtk_widget_get_window (toplevel)))
+        result[n++] = g_strdup_printf ("WINDOWID=%ld", (glong) gdk_x11_window_get_xid (gtk_widget_get_window (toplevel)));
 #endif
 
       /* determine the DISPLAY value for the command */
@@ -957,7 +940,7 @@ terminal_screen_update_colors (TerminalScreen *screen)
   /* cursor color */
   has_cursor = terminal_preferences_get_color (screen->preferences, "color-cursor", &cursor);
   vte_terminal_set_color_cursor (VTE_TERMINAL (screen->terminal), has_cursor ? &cursor : NULL);
-#if VTE_CHECK_VERSION (0, 44, 2)
+#if VTE_CHECK_VERSION (0, 44, 0)
   vte_terminal_set_color_cursor_foreground (VTE_TERMINAL (screen->terminal),
                                             has_cursor && has_bg ? &bg : NULL);
 #endif
@@ -1033,6 +1016,7 @@ terminal_screen_update_misc_bell (TerminalScreen *screen)
   gboolean bval;
   g_object_get (G_OBJECT (screen->preferences), "misc-bell", &bval, NULL);
   vte_terminal_set_audible_bell (VTE_TERMINAL (screen->terminal), bval);
+  g_signal_connect (screen->terminal, "bell", G_CALLBACK (terminal_screen_urgent_bell), screen);
 }
 
 
@@ -1179,8 +1163,9 @@ terminal_screen_update_word_chars (TerminalScreen *screen)
   g_object_get (G_OBJECT (screen->preferences), "word-chars", &word_chars, NULL);
   if (G_LIKELY (word_chars != NULL))
     {
-      // TODO: removed functionality, remove
-      //vte_terminal_set_word_chars (VTE_TERMINAL (screen->terminal), word_chars);
+#if VTE_CHECK_VERSION (0, 40, 0)
+      vte_terminal_set_word_char_exceptions (VTE_TERMINAL (screen->terminal), word_chars);
+#endif
       g_free (word_chars);
     }
 }
@@ -1500,10 +1485,10 @@ terminal_screen_update_label_orientation (TerminalScreen *screen)
 
 
 
-static
-gchar* terminal_screen_zoom_font (TerminalScreen *screen,
-                                  gchar *font_name,
-                                  TerminalZoomLevel zoom)
+static gchar*
+terminal_screen_zoom_font (TerminalScreen *screen,
+                           gchar *font_name,
+                           TerminalZoomLevel zoom)
 {
   gdouble               scale;
   PangoFontDescription *font_desc;
@@ -1556,6 +1541,26 @@ gchar* terminal_screen_zoom_font (TerminalScreen *screen,
 
 
 
+static void
+terminal_screen_urgent_bell (TerminalWidget *widget,
+                             TerminalScreen *screen)
+{
+  GtkWidget *toplevel;
+  gboolean   enabled;
+
+  terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
+
+  g_object_get (G_OBJECT (screen->preferences), "misc-bell-urgent", &enabled, NULL);
+
+  if (!enabled)
+    return;
+
+  toplevel = gtk_widget_get_toplevel (GTK_WIDGET (screen));
+  gtk_window_set_urgency_hint (GTK_WINDOW (toplevel), TRUE);
+}
+
+
+
 /**
  * terminal_screen_launch_child:
  * @screen  : A #TerminalScreen.
@@ -1650,6 +1655,20 @@ terminal_screen_set_custom_command (TerminalScreen *screen,
 
 
 
+/**
+ * terminal_screen_get_custom_title:
+ * @screen  : A #TerminalScreen.
+ **/
+const gchar *
+terminal_screen_get_custom_title (TerminalScreen *screen)
+{
+  terminal_return_val_if_fail (TERMINAL_IS_SCREEN (screen), NULL);
+
+  return screen->custom_title;
+}
+
+
+
 /**
  * terminal_screen_set_custom_title:
  * @screen  : A #TerminalScreen.
@@ -1801,10 +1820,6 @@ terminal_screen_force_resize_window (TerminalScreen *screen,
   glong          char_width;
   glong          char_height;
 
-  // Don't need this on gtk>=3.20
-#if GTK_CHECK_VERSION (3,20,0)
-  return;
-#endif
 
   terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
   terminal_return_if_fail (VTE_IS_TERMINAL (screen->terminal));
@@ -1812,8 +1827,8 @@ terminal_screen_force_resize_window (TerminalScreen *screen,
 
   terminal_screen_set_window_geometry_hints (screen, window);
 
-  gtk_widget_get_preferred_size (GTK_WIDGET (window), &window_requisition, NULL);
-  gtk_widget_get_preferred_size (screen->terminal, &terminal_requisition, NULL);
+  gtk_widget_get_preferred_size (GTK_WIDGET (window), NULL, &window_requisition);
+  gtk_widget_get_preferred_size (screen->terminal, NULL, &terminal_requisition);
 
   if (columns < 1)
     columns = vte_terminal_get_column_count (VTE_TERMINAL (screen->terminal));
@@ -1838,8 +1853,6 @@ terminal_screen_force_resize_window (TerminalScreen *screen,
     gtk_window_resize (window, width, height);
   else
     gtk_window_set_default_size (window, width, height);
-
-  gtk_widget_grab_focus (screen->terminal);
 }
 
 
@@ -1943,7 +1956,7 @@ terminal_screen_get_working_directory (TerminalScreen *screen)
       file = g_strdup_printf ("/proc/%d/cwd", screen->pid);
 #endif
 
-      length = readlink (file, buffer, sizeof (buffer));
+      length = readlink (file, buffer, sizeof (buffer) - 1);
       if (length > 0 && *buffer == '/')
         {
           buffer[length] = '\0';
@@ -2208,7 +2221,7 @@ terminal_screen_get_tab_label (TerminalScreen *screen)
                             G_CALLBACK (gtk_widget_destroy), screen);
 
   /* button image */
-  image = gtk_image_new_from_icon_name ("window-close", GTK_ICON_SIZE_MENU);
+  image = gtk_image_new_from_icon_name ("window-close-symbolic", GTK_ICON_SIZE_MENU);
   gtk_container_add (GTK_CONTAINER (button), image);
 
   /* show the box and all its widgets */
@@ -2294,3 +2307,25 @@ terminal_screen_search_find_previous (TerminalScreen *screen)
   terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
   vte_terminal_search_find_previous (VTE_TERMINAL (screen->terminal));
 }
+
+
+
+void
+terminal_screen_set_input_enabled (TerminalScreen *screen,
+                                   gboolean        enabled)
+{
+  terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
+  vte_terminal_set_input_enabled (VTE_TERMINAL (screen->terminal), enabled);
+}
+
+
+
+void
+terminal_screen_save_contents (TerminalScreen *screen,
+                               GOutputStream  *stream,
+                               GError         *error)
+{
+  terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
+  vte_terminal_write_contents_sync (VTE_TERMINAL (screen->terminal),
+                                    stream, VTE_WRITE_DEFAULT, NULL, &error);
+}
